@@ -15,26 +15,28 @@
 //! 1. DAG vertices are integer numbers (`usize`) which is used to trivially
 //!    test whether adding an edge would form a cycle.  It is simply stipulated
 //!    that an edge can only go from a node `u` to a node `v` when `u < v`.
-//!    Otherwise we panic.
+//!    Otherwise we panic with a [`debug_assert`].
 //! 1. Vertices numbering starts at 0.
 //! 1. The number of vertices is determined at construction time and
 //!    growing/shrinking generally requires a new graph to be constructed.
 //!
 //! In exchange for these assumptions you get these useful properties:
-//! * It's not possible to represent a graph with the [`DirectedAcyclicGraph`]
-//!   data type that's not a DAG, contrary to a fully general graph
-//!   representation like adjacency lists or a square matrix.  IOW: Every
-//!   strictly upper triangular matrix represents *some* valid DAG.  At the same
-//!   time, every DAG is represented by some strictly upper triangular matrix.
-//! * The representation is *compact*: edges are just bits in a bit set.  The
-//!   implementation uses just (n*n-n)/2 bits + O(1) memory for other fields,
-//!   where n is the number of vertices. Iteration over the edges of some vertex
-//!   is just iteration over bits in a bit set, so it's CPU-cache-friendly.
-//!   That's nod at [Data Oriented
-//!   Design](https://en.wikipedia.org/wiki/Data-oriented_design).
-//! * Generating a random DAG is a linear operation, contrary to a fully general
-//!   graph representation.  That was actually the original motivation for
-//!   writing this crate.  It can be used with
+//! * **Correctness**: It's not possible to represent a graph with the
+//!   [`DirectedAcyclicGraph`] data type that's not a DAG, contrary to a fully
+//!   general graph representation like adjacency lists or a square matrix. IOW:
+//!   Every strictly upper triangular matrix represents *some* valid DAG. At the
+//!   same time, every DAG is represented by some strictly upper triangular
+//!   matrix.
+//! * **Efficiency**: The representation is *compact*: edges are just bits in a
+//!   bit set.  The implementation uses just `(n*n-n)/2` *bits* of memory + a
+//!   constant, where `n` is the number of vertices.
+//! * **Efficiency**: The chosen matrix representation is a [row-major packed
+//!   representation](https://www.intel.com/content/www/us/en/develop/documentation/onemkl-developer-reference-c/top/lapack-routines/matrix-storage-schemes-for-lapack-routines.html)
+//!   so that iteration over the edges of a vertex is just an iteration over
+//!   *consecutive* bits in a bit set, so it has good CPU cache locality.
+//! * **Efficiency**: Generating a random DAG is a linear operation, contrary to
+//!   a fully general graph representation.  That was actually the original
+//!   motivation for writing this crate.  It can be used with
 //!   [quickcheck](https://crates.io/crates/quickcheck) efficiently.  In fact,
 //!   [`DirectedAcyclicGraph`] implements [`quickcheck::Arbitrary`] (with
 //!   meaningful shrinking).
@@ -58,14 +60,14 @@ use fixedbitset::FixedBitSet;
 #[cfg(feature = "qc")]
 use quickcheck::{Arbitrary, Gen};
 
-mod strictly_upper_triangular_matrix;
-pub use strictly_upper_triangular_matrix::EdgesIterator;
-pub use strictly_upper_triangular_matrix::NeighboursIterator;
-pub use strictly_upper_triangular_matrix::StrictlyUpperTriangularMatrix;
+mod strictly_upper_triangular_logical_matrix;
+pub use strictly_upper_triangular_logical_matrix::EdgesIterator;
+pub use strictly_upper_triangular_logical_matrix::NeighboursIterator;
+pub use strictly_upper_triangular_logical_matrix::StrictlyUpperTriangularLogicalMatrix;
 
 #[derive(Clone)]
 pub struct DirectedAcyclicGraph {
-    adjacency_matrix: StrictlyUpperTriangularMatrix,
+    adjacency_matrix: StrictlyUpperTriangularLogicalMatrix,
 }
 
 impl std::fmt::Debug for DirectedAcyclicGraph {
@@ -82,7 +84,7 @@ impl std::fmt::Debug for DirectedAcyclicGraph {
 }
 
 pub struct ReverseTopologicalOrderVerticesIterator<'a> {
-    adjacency_matrix: &'a StrictlyUpperTriangularMatrix,
+    adjacency_matrix: &'a StrictlyUpperTriangularLogicalMatrix,
     visited: FixedBitSet,
     to_visit: Vec<usize>,
 }
@@ -121,7 +123,7 @@ impl<'a> Iterator for ReverseTopologicalOrderVerticesIterator<'a> {
 }
 
 pub struct PosetPairsIterator<'a> {
-    adjacency_matrix: &'a StrictlyUpperTriangularMatrix,
+    adjacency_matrix: &'a StrictlyUpperTriangularLogicalMatrix,
     inner: ReverseTopologicalOrderVerticesIterator<'a>,
     seen_vertices: FixedBitSet,
     buffer: VecDeque<(usize, usize)>,
@@ -151,37 +153,38 @@ impl<'a> Iterator for PosetPairsIterator<'a> {
 impl DirectedAcyclicGraph {
     pub fn empty(vertex_count: usize) -> Self {
         Self {
-            adjacency_matrix: StrictlyUpperTriangularMatrix::zeroed(vertex_count),
+            adjacency_matrix: StrictlyUpperTriangularLogicalMatrix::zeroed(vertex_count),
         }
     }
 
     /// Constructs a DAG from a list of edges.
     ///
     /// Requires `u < vertex_count && v < vertex_count && u < v` for every edge
-    /// `(u, v)` in `edges`.  Panics otherwise.
+    /// `(u, v)` in `edges`.  Panics with [`debug_assert`] otherwise.
     pub fn from_edges(vertex_count: usize, edges: &[(usize, usize)]) -> Self {
         Self {
-            adjacency_matrix: StrictlyUpperTriangularMatrix::from_ones(vertex_count, edges),
+            adjacency_matrix: StrictlyUpperTriangularLogicalMatrix::from_ones(vertex_count, edges),
         }
     }
 
+    #[inline]
     pub fn get_vertex_count(&self) -> usize {
         self.adjacency_matrix.size()
     }
 
-    /// Requires `u < v`.  Panics otherwise.
+    /// Requires `u < v`.  Panics with [`debug_assert`] otherwise.
     pub fn get_edge(&self, u: usize, v: usize) -> bool {
-        assert!(u < self.get_vertex_count());
-        assert!(v < self.get_vertex_count());
-        assert!(u < v);
+        debug_assert!(u < self.get_vertex_count());
+        debug_assert!(v < self.get_vertex_count());
+        debug_assert!(u < v);
         self.adjacency_matrix.get(u, v)
     }
 
-    /// Requires `u < v`.  Panics otherwise.
+    /// Requires `u < v`.  Panics with [`debug_assert`] otherwise.
     pub fn set_edge(&mut self, u: usize, v: usize, exists: bool) {
-        assert!(u < self.get_vertex_count());
-        assert!(v < self.get_vertex_count());
-        assert!(u < v);
+        debug_assert!(u < self.get_vertex_count());
+        debug_assert!(v < self.get_vertex_count());
+        debug_assert!(u < v);
         self.adjacency_matrix.set(u, v, exists);
     }
 
@@ -195,25 +198,6 @@ impl DirectedAcyclicGraph {
         self.adjacency_matrix.iter_neighbours(v)
     }
 
-    /// Outputs the DAG in the [Graphviz DOT](https://graphviz.org/) format.
-    pub fn to_dot<W: Write>(&self, output: &mut W) -> std::result::Result<(), std::io::Error> {
-        writeln!(output, "digraph dag_{} {{", self.get_vertex_count())?;
-
-        let elements: Vec<usize> = (0..self.get_vertex_count()).collect();
-        for elem in elements {
-            writeln!(output, "\t_{}[label=\"{}\"];", elem, elem)?;
-        }
-
-        writeln!(output, "\n")?;
-
-        for (left, right) in self.iter_edges() {
-            writeln!(output, "\t_{} -> _{};", left, right)?;
-        }
-
-        writeln!(output, "}}")?;
-        Ok(())
-    }
-
     /// Note that the order of the vertices is reverse topological one.
     pub fn iter_reachable_vertices_starting_at(
         &self,
@@ -264,6 +248,27 @@ impl DirectedAcyclicGraph {
     }
 }
 
+/// Outputs the DAG in the [Graphviz DOT](https://graphviz.org/) format.
+pub fn to_dot<W: Write>(
+    dag: &DirectedAcyclicGraph,
+    output: &mut W,
+) -> std::result::Result<(), std::io::Error> {
+    writeln!(output, "digraph dag_{} {{", dag.get_vertex_count())?;
+
+    for elem in 0..dag.get_vertex_count() {
+        writeln!(output, "\t_{}[label=\"{}\"];", elem, elem)?;
+    }
+
+    writeln!(output, "\n")?;
+
+    for (left, right) in dag.iter_edges() {
+        writeln!(output, "\t_{} -> _{};", left, right)?;
+    }
+
+    writeln!(output, "}}")?;
+    Ok(())
+}
+
 /// Returns a new DAG that is a [transitive
 /// reduction](https://en.wikipedia.org/wiki/Transitive_reduction) of a DAG.
 pub fn transitive_reduction(dag: &DirectedAcyclicGraph) -> DirectedAcyclicGraph {
